@@ -1,34 +1,36 @@
 context("Tests DHARMa functions on all implemented model types")
 
+library(MASS)
 library(lme4)
 
-runEverything = function(fittedModel, testData){
+
+runEverything = function(fittedModel, testData, DHARMaData = T){
   simulationOutput <- simulateResiduals(fittedModel = fittedModel)
-  
+
   print(simulationOutput)
   plot(simulationOutput, quantreg = F)
-  
+
   plotSimulatedResiduals(simulationOutput = simulationOutput)
-  plotResiduals(pred = testData$Environment1, simulationOutput$scaledResiduals, quantreg = F)
+  if(DHARMaData == T) plotResiduals(pred = testData$Environment1, simulationOutput$scaledResiduals, quantreg = F)
 
   testUniformity(simulationOutput = simulationOutput)
   testZeroInflation(simulationOutput = simulationOutput)
   testTemporalAutocorrelation(simulationOutput = simulationOutput, time = runif(length(simulationOutput$scaledResiduals )))
   testSpatialAutocorrelation(simulationOutput = simulationOutput, x = runif(length(simulationOutput$scaledResiduals )), y =  runif(length(simulationOutput$scaledResiduals )))
 
-  
+
   simulationOutput2 <- simulateResiduals(fittedModel = fittedModel, refit = T, n = 10) # n=10 is very low, set higher for serious tests
-  
+
   print(simulationOutput2)
   plot(simulationOutput2, quantreg = F)
-  
+
   testOverdispersion(simulationOutput2)
   testOverdispersionParametric(fittedModel)
-  
+
 }
 
 
-test_that("lm gaussian works", 
+test_that("lm gaussian works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 100, overdispersion = 0, randomEffectVariance = 0, family = gaussian())
@@ -38,7 +40,7 @@ test_that("lm gaussian works",
 )
 
 
-test_that("glm gaussian works", 
+test_that("glm gaussian works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 200, overdispersion = 0, randomEffectVariance = 0, family = gaussian())
@@ -48,7 +50,7 @@ test_that("glm gaussian works",
 )
 
 
-test_that("lmer gaussian works", 
+test_that("lmer gaussian works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 100, overdispersion = 0, randomEffectVariance = 1, family = gaussian())
@@ -58,7 +60,7 @@ test_that("lmer gaussian works",
 )
 
 
-test_that("glm binomial 1/0 works", 
+test_that("glm binomial 1/0 works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 200, overdispersion = 0, randomEffectVariance = 0, family = binomial())
@@ -68,7 +70,7 @@ test_that("glm binomial 1/0 works",
 )
 
 
-test_that("glm binomial n/k works", 
+test_that("glm binomial n/k works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 200, overdispersion = 0, randomEffectVariance = 0, family = binomial(), binomialTrials = 20)
@@ -79,10 +81,10 @@ test_that("glm binomial n/k works",
 
 
 
-test_that("glmer binomial 1/0 works", 
+test_that("glmer binomial 1/0 works",
           {
             skip_on_cran()
-            
+
             testData = createData(sampleSize = 200, overdispersion = 2, family = binomial())
             fittedModel <- glmer(observedResponse ~ Environment1 + (1|group) , family = "binomial", data = testData)
             runEverything(fittedModel, testData)
@@ -90,10 +92,10 @@ test_that("glmer binomial 1/0 works",
 )
 
 
-test_that("glmer binomial n/k works", 
+test_that("glmer binomial n/k works",
           {
             skip_on_cran()
-            
+
             testData = createData(sampleSize = 200, overdispersion = 2, family = binomial(), binomialTrials = 20)
             fittedModel <- glmer(cbind(observedResponse1,observedResponse0) ~ Environment1 + (1|group) , family = "binomial", data = testData)
             runEverything(fittedModel, testData)
@@ -101,7 +103,7 @@ test_that("glmer binomial n/k works",
 )
 
 
-test_that("glm poisson works", 
+test_that("glm poisson works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 200, overdispersion = 0.5, randomEffectVariance = 1, family = poisson(), roundPoissonVariance = 0.1, pZeroInflation = 0.1)
@@ -112,7 +114,7 @@ test_that("glm poisson works",
 
 
 
-test_that("glmer poisson works", 
+test_that("glmer poisson works",
           {
             skip_on_cran()
             testData = createData(sampleSize = 200, overdispersion = 0.5, randomEffectVariance = 1, family = poisson(), roundPoissonVariance = 0.1, pZeroInflation = 0.1)
@@ -122,3 +124,33 @@ test_that("glmer poisson works",
 )
 
 
+# Negative binomial models
+
+
+test_that("glmer.nb works",
+          {
+            skip_on_cran()
+
+            testData = createData(sampleSize = 200, randomEffectVariance = 1, family = negative.binomial(theta = 1.2, link = "log"))
+
+
+            fittedModel <- glmer.nb(observedResponse ~ Environment1 + (1|group) , data = testData, control=glmerControl(optCtrl=list(maxfun=20000) ))
+
+            runEverything(fittedModel, testData)
+          }
+)
+
+
+
+test_that("glm.nb from MASS works",
+          {
+            skip_on_cran()
+
+            testData = createData(sampleSize = 200, randomEffectVariance = 1, family = negative.binomial(theta = 1.2, link = "log"))
+
+            fittedModel <- glm.nb(observedResponse ~ Environment1,  data = testData)
+            runEverything(fittedModel, testData)
+          }
+)
+
+
